
#include "DDSDualModule.h"


// Globals for the command interpreter.
String g_command      = "";
const String MHZ      = "mhz";
const String KHZ      = "khz";
const String HZ       = "hz";

DDSDualModule * g_dualDDS;

void ProcessCommand() {
    
    // Parse to lower case so that it makes commands easier to deal with.
    g_command.toLowerCase();  
    
    // Echo back the command as feedback to the user.
    Serial.println(g_command);
    
    if (g_command == "?") {
        PrintHelp();
    } else if (g_command == "pins") {
        PrintPins();
    } else if (g_command == "as") {
        g_dualDDS->SetASin();       
    } else if (g_command == "ac") {
        g_dualDDS->SetACos();
    } else if (g_command == "bs") {
        g_dualDDS->SetBSin();
    } else if (g_command == "bc") {
        g_dualDDS->SetBCos();
    } else if (g_command == "g" || g_command == "w") {
        g_dualDDS->WriteControlBits();
    } else if (g_command.endsWith(MHZ)) {
        g_command.replace(MHZ,"");    // strip out MHZ chars, leaving just numerics
        if (IsNumeric(g_command)) {         
            unsigned long freq = g_command.toFloat() * 1000000;      
            g_dualDDS->SetFreq(freq);
        }
    } else if (g_command.endsWith(KHZ)) {
        g_command.replace(KHZ,"");     
        if (IsNumeric(g_command)) {
            unsigned long freq = g_command.toFloat() * 1000; 
            g_dualDDS->SetFreq(freq);
        }
    } else if (g_command.endsWith(HZ)) {                  
        g_command.replace(HZ,"");
        if (IsNumeric(g_command)) {
            const int bufSize = 10;
            char buf[bufSize];
            g_command.toCharArray(buf, bufSize);
            unsigned long freq = atol(buf);
            g_dualDDS->SetFreq(freq);
        }
    }
    g_command = "";
}

// Returns true if the string is a number x.y or x
boolean IsNumeric(String str) {
    for(char i = 0; i < str.length(); i++) {
        if ( !(isDigit(str.charAt(i)) || str.charAt(i) == '.' )) {
            return false;
        }
    }
   return true;
}

// Prints out all the commands that are available to the user.
void PrintHelp() {
    Serial.println("COMMAND LIST");   
    Serial.println("as      DDS A sine o/p ");
    Serial.println("ac      DDS A cos o/p");
    Serial.println("bs      DDS B sine o/p");
    Serial.println("bc      DDS B cos o/p");
    Serial.println("100Hz   example");
    Serial.println("100kHz  example");
    Serial.println("1MHz    example");
    Serial.println("pins    list pins");
    Serial.println();
}

// Prints the pin numbers to the user, used in case they need to check something.
void PrintPins() {
     Serial.println("Nano pins");   
     Serial.println("DDS_ResetPin        = 4");
     Serial.println("DDS_DataPin         = 5");
     Serial.println("DDS_FreqUpdatePin   = 6");
     Serial.println("DDSA_DataClockPin   = 7");
     Serial.println("DDSB_DataClockPin   = 8"); 
     Serial.println();
}

// Initialisation and setup code, only gets executed once.
void setup() {
  
    // Pins for the DDS modules.
    const int DDS_ResetPin = 4;
    const int DDS_DataPin = 5;
    const int DDS_FreqUpdatePin = 6; 
    const int DDSA_DataClockPin = 7;
    const int DDSB_DataClockPin = 8;

    // Initialise the serial port.
    Serial.begin(9600);
    delay(500);
    Serial.println("Starting");
    
    PrintHelp();

    // Instantiate a DDSDualModule with the correct pins.
    g_dualDDS = new DDSDualModule(DDS_ResetPin, DDS_FreqUpdatePin, DDS_DataPin, DDSA_DataClockPin, DDSB_DataClockPin);  
}

// Main code loop
void loop() {
   
   // Read the incoming byte from the serial port which will make up the commands to interpret.
   if (Serial.available() > 0) {
        char incomingByte;
        incomingByte = Serial.read();

        // If the character is not a new line then build up command string, else process what ever is in g_command.
        if (incomingByte != '\n' && incomingByte != '\r') {
            g_command += incomingByte; 
        } else {
            ProcessCommand();
        }
    }
}
